@@ -25,12 +25,15 @@ import (
 )
 
 var (
-	pruneDryRunArg      bool
-	pruneVerboseArg     bool
-	pruneVerifyArg      bool
-	pruneRecentArg      bool
-	pruneForceArg       bool
-	pruneDoNotVerifyArg bool
+	pruneDryRunArg                 bool
+	pruneVerboseArg                bool
+	pruneVerifyArg                 bool
+	pruneRecentArg                 bool
+	pruneForceArg                  bool
+	pruneDoNotVerifyArg            bool
+	pruneVerifyUnreachableArg      bool
+	pruneDoNotVerifyUnreachableArg bool
+	pruneWhenUnverifiedArg         string
 )
 
 func pruneCommand(cmd *cobra.Command, args []string) {
@@ -42,17 +45,30 @@ func pruneCommand(cmd *cobra.Command, args []string) {
 	fetchPruneConfig := lfs.NewFetchPruneConfig(cfg.Git)
 	verify := !pruneDoNotVerifyArg &&
 		(fetchPruneConfig.PruneVerifyRemoteAlways || pruneVerifyArg)
+	verifyUnreachable := !pruneDoNotVerifyUnreachableArg && (pruneVerifyUnreachableArg || fetchPruneConfig.PruneVerifyUnreachableAlways)
+
+	continueWhenUnverified := false
+	switch pruneWhenUnverifiedArg {
+	case "halt":
+		continueWhenUnverified = false
+	case "continue":
+		continueWhenUnverified = true
+	default:
+		Exit(tr.Tr.Get("Invalid value for --when-unverified: %s", pruneWhenUnverifiedArg))
+	}
+
 	fetchPruneConfig.PruneRecent = pruneRecentArg || pruneForceArg
 	fetchPruneConfig.PruneForce = pruneForceArg
-	prune(fetchPruneConfig, verify, pruneDryRunArg, pruneVerboseArg)
+	prune(fetchPruneConfig, verify, verifyUnreachable, continueWhenUnverified, pruneDryRunArg, pruneVerboseArg)
 }
 
 type PruneProgressType int
 
 const (
-	PruneProgressTypeLocal  = PruneProgressType(iota)
-	PruneProgressTypeRetain = PruneProgressType(iota)
-	PruneProgressTypeVerify = PruneProgressType(iota)
+	PruneProgressTypeLocal      = PruneProgressType(iota)
+	PruneProgressTypeRetain     = PruneProgressType(iota)
+	PruneProgressTypeVerify     = PruneProgressType(iota)
+	PruneProgressTypeUnverified = PruneProgressType(iota)
 )
 
 // Progress from a sub-task of prune
@@ -62,7 +78,7 @@ type PruneProgress struct {
 }
 type PruneProgressChan chan PruneProgress
 
-func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose bool) {
+func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, verifyUnreachable, continueWhenUnverified, dryRun, verbose bool) {
 	localObjects := make([]fs.Object, 0, 100)
 	retainedObjects := tools.NewStringSetWithCapacity(100)
 
@@ -78,7 +94,7 @@ func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose
 	// one completes really fast & hits 0 unexpectedly
 	// each main process can Add() to the wg itself if it subdivides the task
 	taskwait.Add(5) // 1..5: localObjects, current & recent refs, unpushed, worktree, stashes
-	if verifyRemote {
+	if verifyRemote && !verifyUnreachable {
 		taskwait.Add(1) // 6
 	}
 
@@ -106,7 +122,7 @@ func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose
 	go pruneTaskGetRetainedUnpushed(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait, sem)
 	go pruneTaskGetRetainedWorktree(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait, sem)
 	go pruneTaskGetRetainedStashed(gitscanner, retainChan, errorChan, &taskwait, sem)
-	if verifyRemote {
+	if verifyRemote && !verifyUnreachable {
 		reachableObjects = tools.NewStringSetWithCapacity(100)
 		go pruneTaskGetReachableObjects(gitscanner, &reachableObjects, errorChan, &taskwait, sem)
 	}
@@ -121,8 +137,7 @@ func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose
 	progresswait.Add(1)
 	go pruneTaskDisplayProgress(progressChan, &progresswait, logger)
 
-	taskwait.Wait() // wait for subtasks
-	gitscanner.Close()
+	taskwait.Wait()   // wait for subtasks
 	close(retainChan) // triggers retain collector to end now all tasks have
 	retainwait.Wait() // make sure all retained objects added
 
@@ -173,8 +188,6 @@ func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose
 			}
 
 			if verifyRemote {
-				tracerx.Printf("VERIFYING: %v", file.Oid)
-
 				verifyQueue.Add(downloadTransfer(&lfs.WrappedPointer{
 					Pointer: lfs.NewPointer(file.Oid, file.Size, nil),
 				}))
@@ -185,9 +198,20 @@ func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose
 	if verifyRemote {
 		verifyQueue.Wait()
 		verifywait.Wait()
+
+		var problems bytes.Buffer
+		prunableObjectsLen := len(prunableObjects)
+		prunableObjects, problems = pruneGetVerifiedPrunableObjects(prunableObjects, reachableObjects, verifiedObjects, verifyUnreachable)
+		if prunableObjectsLen != len(prunableObjects) {
+			progressChan <- PruneProgress{PruneProgressTypeUnverified, prunableObjectsLen - len(prunableObjects)}
+		}
+
 		close(progressChan) // after verify but before check
 		progresswait.Wait()
-		pruneCheckVerified(prunableObjects, reachableObjects, verifiedObjects)
+
+		if !continueWhenUnverified && problems.Len() > 0 {
+			Exit("%s\n%v", tr.Tr.Get("These objects to be pruned are missing on remote:"), problems.String())
+		}
 	} else {
 		close(progressChan)
 		progresswait.Wait()
@@ -197,50 +221,59 @@ func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose
 		return
 	}
 
-	info := tasklog.NewSimpleTask()
-	logger.Enqueue(info)
-	if dryRun {
-		info.Logf("prune: %s", tr.Tr.GetN(
-			"%d file would be pruned (%s)",
-			"%d files would be pruned (%s)",
-			len(prunableObjects),
-			len(prunableObjects),
-			humanize.FormatBytes(uint64(totalSize))))
-		for _, item := range verboseOutput {
-			info.Logf("\n * %s", item)
-		}
-		info.Complete()
-	} else {
-		for _, item := range verboseOutput {
-			info.Logf("\n%s", item)
-		}
-		info.Complete()
+	logVerboseOutput(logger, verboseOutput, len(prunableObjects), totalSize, dryRun)
 
+	if !dryRun {
 		pruneDeleteFiles(prunableObjects, logger)
 	}
 }
 
-func pruneCheckVerified(prunableObjects []string, reachableObjects, verifiedObjects tools.StringSet) {
-	// There's no issue if an object is not reachable and missing, only if reachable & missing
-	var problems bytes.Buffer
+func logVerboseOutput(logger *tasklog.Logger, verboseOutput []string, numPrunableObjects int, totalSize int64, dryRun bool) {
+	info := logger.Simple()
+	defer info.Complete()
+
+	if dryRun {
+		info.Logf("prune: %s", tr.Tr.GetN(
+			"%d file would be pruned (%s)",
+			"%d files would be pruned (%s)",
+			numPrunableObjects,
+			numPrunableObjects,
+			humanize.FormatBytes(uint64(totalSize))))
+		for _, item := range verboseOutput {
+			info.Logf("\n * %s", item)
+		}
+	} else {
+		for _, item := range verboseOutput {
+			info.Logf("\n%s", item)
+		}
+	}
+}
+
+func pruneGetVerifiedPrunableObjects(prunableObjects []string, reachableObjects, verifiedObjects tools.StringSet, verifyUnreachable bool) ([]string, bytes.Buffer) {
+	verifiedPrunableObjects := make([]string, 0, len(verifiedObjects))
+	var unverified bytes.Buffer
+
 	for _, oid := range prunableObjects {
-		// Test verified first as most likely reachable
-		if !verifiedObjects.Contains(oid) {
-			if reachableObjects.Contains(oid) {
-				problems.WriteString(fmt.Sprintf(" * %v\n", oid))
+		if verifiedObjects.Contains(oid) {
+			verifiedPrunableObjects = append(verifiedPrunableObjects, oid)
+		} else {
+			if verifyUnreachable {
+				tracerx.Printf("UNVERIFIED: %v", oid)
+				unverified.WriteString(fmt.Sprintf(" * %v\n", oid))
 			} else {
-				// Just to indicate why it doesn't matter that we didn't verify
-				tracerx.Printf("UNREACHABLE: %v", oid)
+				// There's no issue if an object is not reachable and missing, only if reachable & missing
+				if reachableObjects.Contains(oid) {
+					unverified.WriteString(fmt.Sprintf(" * %v\n", oid))
+				} else {
+					// Just to indicate why it doesn't matter that we didn't verify
+					tracerx.Printf("UNREACHABLE: %v", oid)
+					verifiedPrunableObjects = append(verifiedPrunableObjects, oid)
+				}
 			}
 		}
 	}
-	// technically we could still prune the other oids, but this indicates a
-	// more serious issue because the local state implies that these can be
-	// deleted but that's incorrect; bad state has occurred somehow, might need
-	// push --all to resolve
-	if problems.Len() > 0 {
-		Exit("%s\n%v", tr.Tr.Get("These objects to be pruned are missing on remote:"), problems.String())
-	}
+
+	return verifiedPrunableObjects, unverified
 }
 
 func pruneCheckErrors(taskErrors []error) {
@@ -255,14 +288,13 @@ func pruneCheckErrors(taskErrors []error) {
 func pruneTaskDisplayProgress(progressChan PruneProgressChan, waitg *sync.WaitGroup, logger *tasklog.Logger) {
 	defer waitg.Done()
 
-	task := tasklog.NewSimpleTask()
+	task := logger.Simple()
 	defer task.Complete()
 
-	logger.Enqueue(task)
-
 	localCount := 0
 	retainCount := 0
 	verifyCount := 0
+	notRemoteCount := 0
 	var msg string
 	for p := range progressChan {
 		switch p.ProgressType {
@@ -272,6 +304,8 @@ func pruneTaskDisplayProgress(progressChan PruneProgressChan, waitg *sync.WaitGr
 			retainCount++
 		case PruneProgressTypeVerify:
 			verifyCount++
+		case PruneProgressTypeUnverified:
+			notRemoteCount += p.Count
 		}
 		msg = fmt.Sprintf("prune: %s, %s",
 			tr.Tr.GetN("%d local object", "%d local objects", localCount, localCount),
@@ -279,6 +313,9 @@ func pruneTaskDisplayProgress(progressChan PruneProgressChan, waitg *sync.WaitGr
 		if verifyCount > 0 {
 			msg += tr.Tr.GetN(", %d verified with remote", ", %d verified with remote", verifyCount, verifyCount)
 		}
+		if notRemoteCount > 0 {
+			msg += tr.Tr.GetN(", %d not on remote", ", %d not on remote", notRemoteCount, notRemoteCount)
+		}
 		task.Log(msg)
 	}
 }
@@ -306,6 +343,7 @@ func pruneTaskCollectErrors(outtaskErrors *[]error, errorChan chan error, errorw
 
 func pruneDeleteFiles(prunableObjects []string, logger *tasklog.Logger) {
 	task := logger.Percentage(fmt.Sprintf("prune: %s", tr.Tr.Get("Deleting objects")), uint64(len(prunableObjects)))
+	defer task.Complete()
 
 	var problems bytes.Buffer
 	// In case we fail to delete some
@@ -468,13 +506,9 @@ func pruneTaskGetRetainedUnpushed(gitscanner *lfs.GitScanner, fetchconf lfs.Fetc
 func pruneTaskGetRetainedWorktree(gitscanner *lfs.GitScanner, fetchconf lfs.FetchPruneConfig, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
 	defer waitg.Done()
 
-	if fetchconf.PruneForce {
-		return
-	}
-
 	// Retain other worktree HEADs too
 	// Working copy, branch & maybe commit is different but repo is shared
-	allWorktreeRefs, err := git.GetAllWorkTreeHEADs(cfg.LocalGitStorageDir())
+	allWorktrees, err := git.GetAllWorkTrees(cfg.LocalGitStorageDir())
 	if err != nil {
 		errorChan <- err
 		return
@@ -482,20 +516,29 @@ func pruneTaskGetRetainedWorktree(gitscanner *lfs.GitScanner, fetchconf lfs.Fetc
 	// Don't repeat any commits, worktrees are always on their own branches but
 	// may point to the same commit
 	commits := tools.NewStringSet()
-	// current HEAD is done elsewhere
-	headref, err := git.CurrentRef()
-	if err != nil {
-		errorChan <- err
-		return
+
+	if !fetchconf.PruneForce {
+		// current HEAD is done elsewhere
+		headref, err := git.CurrentRef()
+		if err != nil {
+			errorChan <- err
+			return
+		}
+
+		commits.Add(headref.Sha)
 	}
-	commits.Add(headref.Sha)
-	for _, ref := range allWorktreeRefs {
-		if commits.Add(ref.Sha) {
+
+	for _, worktree := range allWorktrees {
+		if !fetchconf.PruneForce && commits.Add(worktree.Ref.Sha) {
 			// Worktree is on a different commit
 			waitg.Add(1)
 			// Don't need to 'cd' to worktree since we share same repo
-			go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg, sem)
+			go pruneTaskGetRetainedAtRef(gitscanner, worktree.Ref.Sha, retainChan, errorChan, waitg, sem)
 		}
+
+		// Always scan the index of the worktree
+		waitg.Add(1)
+		go pruneTaskGetRetainedIndex(gitscanner, worktree.Ref.Sha, worktree.Dir, retainChan, errorChan, waitg, sem)
 	}
 }
 
@@ -518,6 +561,25 @@ func pruneTaskGetRetainedStashed(gitscanner *lfs.GitScanner, retainChan chan str
 	}
 }
 
+// Background task, must call waitg.Done() once at end
+func pruneTaskGetRetainedIndex(gitscanner *lfs.GitScanner, ref string, workingDir string, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
+	defer waitg.Done()
+
+	err := gitscanner.ScanIndex(ref, workingDir, func(p *lfs.WrappedPointer, err error) {
+		if err != nil {
+			errorChan <- err
+		} else {
+			retainChan <- p.Pointer.Oid
+			tracerx.Printf("RETAIN: %v index", p.Pointer.Oid)
+		}
+	})
+
+	if err != nil {
+		errorChan <- err
+		return
+	}
+}
+
 // Background task, must call waitg.Done() once at end
 func pruneTaskGetReachableObjects(gitscanner *lfs.GitScanner, outObjectSet *tools.StringSet, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
 	defer waitg.Done()
@@ -544,7 +606,10 @@ func init() {
 		cmd.Flags().BoolVarP(&pruneVerboseArg, "verbose", "v", false, "Print full details of what is/would be deleted")
 		cmd.Flags().BoolVarP(&pruneRecentArg, "recent", "", false, "Prune even recent objects")
 		cmd.Flags().BoolVarP(&pruneForceArg, "force", "f", false, "Prune everything that has been pushed")
-		cmd.Flags().BoolVarP(&pruneVerifyArg, "verify-remote", "c", false, "Verify that remote has LFS files before deleting")
+		cmd.Flags().BoolVarP(&pruneVerifyArg, "verify-remote", "c", false, "Verify that remote has reachable LFS files before deleting")
 		cmd.Flags().BoolVar(&pruneDoNotVerifyArg, "no-verify-remote", false, "Override lfs.pruneverifyremotealways and don't verify")
+		cmd.Flags().BoolVar(&pruneVerifyUnreachableArg, "verify-unreachable", false, "When using --verify-remote, additionally verify unreachable LFS files before deleting.")
+		cmd.Flags().BoolVar(&pruneDoNotVerifyUnreachableArg, "no-verify-unreachable", false, "Override lfs.pruneverifyunreachablealways and don't verify unreachable objects")
+		cmd.Flags().StringVar(&pruneWhenUnverifiedArg, "when-unverified", "halt", "halt|continue the execution when objects are not found on the remote")
 	})
 }
