@@ -1,15 +1,15 @@
 package git_test // to avoid import cycles
 
 import (
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"sort"
 	"testing"
 	"time"
 
-	. "github.com/git-lfs/git-lfs/git"
-	test "github.com/git-lfs/git-lfs/t/cmd/util"
+	. "github.com/git-lfs/git-lfs/v3/git"
+	test "github.com/git-lfs/git-lfs/v3/t/cmd/util"
+	"github.com/git-lfs/git-lfs/v3/tools"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -26,11 +26,6 @@ func TestRefString(t *testing.T) {
 			Type: RefTypeRemoteBranch,
 			Sha:  sha,
 		},
-		"refs/remotes/tags/v1.0.0": {
-			Name: "v1.0.0",
-			Type: RefTypeRemoteTag,
-			Sha:  sha,
-		},
 		"refs/tags/v1.0.0": {
 			Name: "v1.0.0",
 			Type: RefTypeLocalTag,
@@ -53,10 +48,9 @@ func TestRefString(t *testing.T) {
 
 func TestParseRefs(t *testing.T) {
 	tests := map[string]RefType{
-		"refs/heads":        RefTypeLocalBranch,
-		"refs/tags":         RefTypeLocalTag,
-		"refs/remotes/tags": RefTypeRemoteTag,
-		"refs/remotes":      RefTypeRemoteBranch,
+		"refs/heads":   RefTypeLocalBranch,
+		"refs/tags":    RefTypeLocalTag,
+		"refs/remotes": RefTypeRemoteBranch,
 	}
 
 	for prefix, expectedType := range tests {
@@ -351,6 +345,8 @@ func TestWorkTrees(t *testing.T) {
 		repo.Cleanup()
 	}()
 
+	repoDir, _ := tools.CanonicalizePath(repo.Path, true)
+
 	// test commits; we'll just modify the same file each time since we're
 	// only interested in branches & dates
 	inputs := []*test.CommitInput{
@@ -390,29 +386,38 @@ func TestWorkTrees(t *testing.T) {
 	test.RunGitCommand(t, true, "worktree", "add", "branch2_wt", "branch2")
 	test.RunGitCommand(t, true, "worktree", "add", "branch4_wt", "branch4")
 
-	refs, err := GetAllWorkTreeHEADs(filepath.Join(repo.Path, ".git"))
+	worktrees, err := GetAllWorkTrees(filepath.Join(repo.Path, ".git"))
 	assert.Equal(t, nil, err)
-	expectedRefs := []*Ref{
+	expectedWorktrees := []*Worktree{
 		{
-			Name: "master",
-			Type: RefTypeLocalBranch,
-			Sha:  outputs[0].Sha,
+			Ref: Ref{
+				Name: "master",
+				Type: RefTypeLocalBranch,
+				Sha:  outputs[0].Sha,
+			},
+			Dir: repoDir,
 		},
 		{
-			Name: "branch2",
-			Type: RefTypeLocalBranch,
-			Sha:  outputs[1].Sha,
+			Ref: Ref{
+				Name: "branch2",
+				Type: RefTypeLocalBranch,
+				Sha:  outputs[1].Sha,
+			},
+			Dir: filepath.Join(repoDir, "branch2_wt"),
 		},
 		{
-			Name: "branch4",
-			Type: RefTypeLocalBranch,
-			Sha:  outputs[3].Sha,
+			Ref: Ref{
+				Name: "branch4",
+				Type: RefTypeLocalBranch,
+				Sha:  outputs[3].Sha,
+			},
+			Dir: filepath.Join(repoDir, "branch4_wt"),
 		},
 	}
 	// Need to sort for consistent comparison
-	sort.Sort(test.RefsByName(expectedRefs))
-	sort.Sort(test.RefsByName(refs))
-	assert.Equal(t, expectedRefs, refs, "Refs should be correct")
+	sort.Sort(test.WorktreesByName(expectedWorktrees))
+	sort.Sort(test.WorktreesByName(worktrees))
+	assert.Equal(t, expectedWorktrees, worktrees, "Worktrees should be correct")
 }
 
 func TestVersionCompare(t *testing.T) {
@@ -524,7 +529,7 @@ func TestGetTrackedFiles(t *testing.T) {
 	assert.Equal(t, []string{"file1.txt", "file2.txt", "file3.txt", "file4.txt"}, tracked)
 
 	// Test includes staged but uncommitted files
-	ioutil.WriteFile("z_newfile.txt", []byte("Hello world"), 0644)
+	os.WriteFile("z_newfile.txt", []byte("Hello world"), 0644)
 	test.RunGitCommand(t, true, "add", "z_newfile.txt")
 	tracked, err = GetTrackedFiles("*.txt")
 	assert.Nil(t, err)
@@ -533,7 +538,7 @@ func TestGetTrackedFiles(t *testing.T) {
 	assert.Equal(t, fulllist, tracked)
 
 	// Test includes modified files (not staged)
-	ioutil.WriteFile("file1.txt", []byte("Modifications"), 0644)
+	os.WriteFile("file1.txt", []byte("Modifications"), 0644)
 	tracked, err = GetTrackedFiles("*.txt")
 	assert.Nil(t, err)
 	sort.Strings(tracked)
@@ -594,8 +599,6 @@ func TestLocalRefs(t *testing.T) {
 			t.Errorf("Local HEAD ref: %v", r)
 		case RefTypeOther:
 			t.Errorf("Stash or unknown ref: %v", r)
-		case RefTypeRemoteBranch, RefTypeRemoteTag:
-			t.Errorf("Remote ref: %v", r)
 		default:
 			actual[r.Name] = true
 		}
@@ -701,7 +704,6 @@ func TestRefTypeKnownPrefixes(t *testing.T) {
 		RefTypeLocalBranch:  {"refs/heads", true},
 		RefTypeRemoteBranch: {"refs/remotes", true},
 		RefTypeLocalTag:     {"refs/tags", true},
-		RefTypeRemoteTag:    {"refs/remotes/tags", true},
 		RefTypeHEAD:         {"", false},
 		RefTypeOther:        {"", false},
 	} {
@@ -711,3 +713,106 @@ func TestRefTypeKnownPrefixes(t *testing.T) {
 		assert.Equal(t, expected.Ok, ok)
 	}
 }
+
+func TestRemoteURLs(t *testing.T) {
+	repo := test.NewRepo(t)
+	repo.Pushd()
+	defer func() {
+		repo.Popd()
+		repo.Cleanup()
+	}()
+	cfg := repo.GitConfig()
+	cfg.SetLocal("remote.foo.url", "https://github.com/git-lfs/git-lfs.git")
+	cfg.SetLocal("remote.bar.url", "https://github.com/git-lfs/wildmatch.git")
+	cfg.SetLocal("remote.bar.pushurl", "https://github.com/git-lfs/pktline.git")
+
+	expected := make(map[string][]string)
+	expected["foo"] = []string{"https://github.com/git-lfs/git-lfs.git"}
+	expected["bar"] = []string{"https://github.com/git-lfs/wildmatch.git"}
+	actual, err := RemoteURLs(false)
+	assert.Nil(t, err)
+	assert.Equal(t, expected, actual)
+
+	expected["bar"] = []string{"https://github.com/git-lfs/pktline.git"}
+	actual, err = RemoteURLs(true)
+	assert.Nil(t, err)
+	assert.Equal(t, expected, actual)
+}
+
+func TestMapRemoteURL(t *testing.T) {
+	repo := test.NewRepo(t)
+	repo.Pushd()
+	defer func() {
+		repo.Popd()
+		repo.Cleanup()
+	}()
+	cfg := repo.GitConfig()
+	cfg.SetLocal("remote.foo.url", "https://github.com/git-lfs/git-lfs.git")
+	cfg.SetLocal("remote.bar.url", "https://github.com/git-lfs/wildmatch.git")
+	cfg.SetLocal("remote.bar.pushurl", "https://github.com/git-lfs/pktline.git")
+
+	tests := []struct {
+		url   string
+		push  bool
+		match bool
+		val   string
+	}{
+		{
+			"https://github.com/git-lfs/git-lfs.git",
+			false,
+			true,
+			"foo",
+		},
+		{
+			"https://github.com/git-lfs/git-lfs.git",
+			true,
+			true,
+			"foo",
+		},
+		{
+			"https://github.com/git-lfs/wildmatch.git",
+			false,
+			true,
+			"bar",
+		},
+		{
+			"https://github.com/git-lfs/pktline.git",
+			true,
+			true,
+			"bar",
+		},
+		{
+			"https://github.com/git-lfs/pktline.git",
+			false,
+			false,
+			"https://github.com/git-lfs/pktline.git",
+		},
+		{
+			"https://github.com/git/git.git",
+			true,
+			false,
+			"https://github.com/git/git.git",
+		},
+	}
+	for _, test := range tests {
+		val, ok := MapRemoteURL(test.url, test.push)
+		assert.Equal(t, ok, test.match)
+		assert.Equal(t, val, test.val)
+	}
+}
+
+func TestIsValidObjectIDLength(t *testing.T) {
+	// Lengths are 40, 64, 39, and 12.
+	assert.Equal(t, HasValidObjectIDLength("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"), true)
+	assert.Equal(t, HasValidObjectIDLength("2222222222222222222222222222222222222222222222222222222222222222"), true)
+	assert.Equal(t, HasValidObjectIDLength("555555555555555555555555555555555555555"), false)
+	assert.Equal(t, HasValidObjectIDLength("0123456789ab"), false)
+}
+
+func TestIsZeroObjectID(t *testing.T) {
+	assert.Equal(t, IsZeroObjectID("0000000000000000000000000000000000000000"), true)
+	assert.Equal(t, IsZeroObjectID("0000000000000000000000000000000000000000000000000000000000000000"), true)
+	assert.Equal(t, IsZeroObjectID("000000000000000000000000000000000000000"), false)
+	assert.Equal(t, IsZeroObjectID("e69de29bb2d1d6434b8b29ae775ad8c2e48c5391"), false)
+	assert.Equal(t, IsZeroObjectID("473a0f4c3be8a93681a267e3b1e9a7dcda1185436fe141f7749120a303721813"), false)
+}
