@@ -20,14 +20,14 @@ func TestSSHGetLFSExeAndArgs(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Path = "user/repo"
 
-	exe, args, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, "git-lfs-authenticate", "download", false)
+	exe, args, _, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, "git-lfs-authenticate", "download", false, "")
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{
 		"user@foo.com",
 		"git-lfs-authenticate user/repo download",
 	}, args)
 
-	exe, args, _ = ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, "git-lfs-authenticate", "upload", false)
+	exe, args, _, _ = ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, "git-lfs-authenticate", "upload", false, "")
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{
 		"user@foo.com",
@@ -45,7 +45,7 @@ func TestSSHGetExeAndArgsSsh(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{"user@foo.com"}, args)
 }
@@ -61,29 +61,72 @@ func TestSSHGetExeAndArgsSshCustomPort(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{"-p", "8888", "user@foo.com"}, args)
 }
 
-func TestSSHGetExeAndArgsSshMultiplexing(t *testing.T) {
+func TestSSHGetExeAndArgsSshNoMultiplexing(t *testing.T) {
 	cli, err := lfshttp.NewClient(lfshttp.NewContext(nil, map[string]string{
 		"GIT_SSH_COMMAND": "",
 		"GIT_SSH":         "",
-	}, nil))
+	}, map[string]string{
+		"lfs.ssh.automultiplex": "false",
+	}))
 	require.Nil(t, err)
 
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, baseargs, needShell, multiplexing := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, true)
-	exe, args := ssh.FormatArgs(exe, baseargs, needShell, multiplexing)
+	exe, baseargs, needShell, multiplexing, controlPath := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, true, "")
+	exe, args := ssh.FormatArgs(exe, baseargs, needShell, multiplexing, controlPath)
 	assert.Equal(t, "ssh", exe)
-	assert.Equal(t, multiplexing, true)
+	assert.Equal(t, false, multiplexing)
+	assert.Equal(t, []string{"user@foo.com"}, args)
+	assert.Empty(t, controlPath)
+}
+
+func TestSSHGetExeAndArgsSshMultiplexingMaster(t *testing.T) {
+	cli, err := lfshttp.NewClient(lfshttp.NewContext(nil, map[string]string{
+		"GIT_SSH_COMMAND": "",
+		"GIT_SSH":         "",
+	}, map[string]string{
+		"lfs.ssh.automultiplex": "true",
+	}))
+	require.Nil(t, err)
+
+	meta := ssh.SSHMetadata{}
+	meta.UserAndHost = "user@foo.com"
+
+	exe, baseargs, needShell, multiplexing, controlPath := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, true, "")
+	exe, args := ssh.FormatArgs(exe, baseargs, needShell, multiplexing, controlPath)
+	assert.Equal(t, "ssh", exe)
+	assert.Equal(t, true, multiplexing)
 	assert.Equal(t, 3, len(args))
-	assert.Equal(t, "-oControlMaster=auto", args[0])
+	assert.Equal(t, "-oControlMaster=yes", args[0])
 	assert.True(t, strings.HasPrefix(args[1], "-oControlPath="))
 	assert.Equal(t, "user@foo.com", args[2])
+	assert.NotEmpty(t, controlPath)
+}
+
+func TestSSHGetExeAndArgsSshMultiplexingExtra(t *testing.T) {
+	cli, err := lfshttp.NewClient(lfshttp.NewContext(nil, map[string]string{
+		"GIT_SSH_COMMAND": "",
+		"GIT_SSH":         "",
+	}, map[string]string{
+		"lfs.ssh.automultiplex": "true",
+	}))
+	require.Nil(t, err)
+
+	meta := ssh.SSHMetadata{}
+	meta.UserAndHost = "user@foo.com"
+
+	exe, baseargs, needShell, multiplexing, controlPath := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, true, "/tmp/lfs/lfs.sock")
+	exe, args := ssh.FormatArgs(exe, baseargs, needShell, multiplexing, controlPath)
+	assert.Equal(t, "ssh", exe)
+	assert.Equal(t, true, multiplexing)
+	assert.Equal(t, []string{"-oControlMaster=no", "-oControlPath=/tmp/lfs/lfs.sock", "user@foo.com"}, args)
+	assert.Equal(t, "/tmp/lfs/lfs.sock", controlPath)
 }
 
 func TestSSHGetExeAndArgsPlink(t *testing.T) {
@@ -98,7 +141,7 @@ func TestSSHGetExeAndArgsPlink(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"user@foo.com"}, args)
 }
@@ -116,7 +159,7 @@ func TestSSHGetExeAndArgsPlinkCustomPort(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-P", "8888", "user@foo.com"}, args)
 }
@@ -135,7 +178,7 @@ func TestSSHGetExeAndArgsPlinkCustomPortExplicitEnvironment(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-P", "8888", "user@foo.com"}, args)
 }
@@ -154,7 +197,7 @@ func TestSSHGetExeAndArgsPlinkCustomPortExplicitEnvironmentPutty(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-P", "8888", "user@foo.com"}, args)
 }
@@ -173,7 +216,7 @@ func TestSSHGetExeAndArgsPlinkCustomPortExplicitEnvironmentSsh(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-p", "8888", "user@foo.com"}, args)
 }
@@ -190,7 +233,7 @@ func TestSSHGetExeAndArgsTortoisePlink(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-batch", "user@foo.com"}, args)
 }
@@ -208,7 +251,7 @@ func TestSSHGetExeAndArgsTortoisePlinkCustomPort(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-batch", "-P", "8888", "user@foo.com"}, args)
 }
@@ -227,7 +270,7 @@ func TestSSHGetExeAndArgsTortoisePlinkCustomPortExplicitEnvironment(t *testing.T
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-batch", "-P", "8888", "user@foo.com"}, args)
 }
@@ -248,7 +291,7 @@ func TestSSHGetExeAndArgsTortoisePlinkCustomPortExplicitConfig(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-batch", "-P", "8888", "user@foo.com"}, args)
 }
@@ -268,7 +311,7 @@ func TestSSHGetExeAndArgsTortoisePlinkCustomPortExplicitConfigOverride(t *testin
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, plink, exe)
 	assert.Equal(t, []string{"-P", "8888", "user@foo.com"}, args)
 }
@@ -284,7 +327,7 @@ func TestSSHGetExeAndArgsSshCommandPrecedence(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd user@foo.com"}, args)
 }
@@ -299,7 +342,7 @@ func TestSSHGetExeAndArgsSshCommandArgs(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd --args 1 user@foo.com"}, args)
 }
@@ -314,7 +357,7 @@ func TestSSHGetExeAndArgsSshCommandArgsWithMixedQuotes(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd foo 'bar \"baz\"' user@foo.com"}, args)
 }
@@ -329,7 +372,7 @@ func TestSSHGetExeAndArgsSshCommandCustomPort(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd -p 8888 user@foo.com"}, args)
 }
@@ -345,7 +388,7 @@ func TestSSHGetExeAndArgsCoreSshCommand(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd --args 2 user@foo.com"}, args)
 }
@@ -359,7 +402,7 @@ func TestSSHGetExeAndArgsCoreSshCommandArgsWithMixedQuotes(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd foo 'bar \"baz\"' user@foo.com"}, args)
 }
@@ -373,7 +416,7 @@ func TestSSHGetExeAndArgsConfigVersusEnv(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", "sshcmd --args 1 user@foo.com"}, args)
 }
@@ -389,7 +432,7 @@ func TestSSHGetExeAndArgsPlinkCommand(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", plink + " user@foo.com"}, args)
 }
@@ -406,7 +449,7 @@ func TestSSHGetExeAndArgsPlinkCommandCustomPort(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", plink + " -P 8888 user@foo.com"}, args)
 }
@@ -422,7 +465,7 @@ func TestSSHGetExeAndArgsTortoisePlinkCommand(t *testing.T) {
 	meta := ssh.SSHMetadata{}
 	meta.UserAndHost = "user@foo.com"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", plink + " -batch user@foo.com"}, args)
 }
@@ -439,7 +482,7 @@ func TestSSHGetExeAndArgsTortoisePlinkCommandCustomPort(t *testing.T) {
 	meta.UserAndHost = "user@foo.com"
 	meta.Port = "8888"
 
-	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false))
+	exe, args := ssh.FormatArgs(ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &meta, false, ""))
 	assert.Equal(t, "sh", exe)
 	assert.Equal(t, []string{"-c", plink + " -batch -P 8888 user@foo.com"}, args)
 }
@@ -460,7 +503,7 @@ func TestSSHGetLFSExeAndArgsWithCustomSSH(t *testing.T) {
 	assert.Equal(t, "git@host.com", e.SSHMetadata.UserAndHost)
 	assert.Equal(t, "/repo", e.SSHMetadata.Path)
 
-	exe, args, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, "git-lfs-authenticate", "download", false)
+	exe, args, _, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, "git-lfs-authenticate", "download", false, "")
 	assert.Equal(t, "not-ssh", exe)
 	assert.Equal(t, []string{"-p", "12345", "git@host.com", "git-lfs-authenticate /repo download"}, args)
 }
@@ -478,7 +521,7 @@ func TestSSHGetLFSExeAndArgsInvalidOptionsAsHost(t *testing.T) {
 	assert.Equal(t, "-oProxyCommand=gnome-calculator", e.SSHMetadata.UserAndHost)
 	assert.Equal(t, "/repo", e.SSHMetadata.Path)
 
-	exe, args, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, "git-lfs-authenticate", "download", false)
+	exe, args, _, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, "git-lfs-authenticate", "download", false, "")
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{"--", "-oProxyCommand=gnome-calculator", "git-lfs-authenticate /repo download"}, args)
 }
@@ -499,7 +542,7 @@ func TestSSHGetLFSExeAndArgsInvalidOptionsAsHostWithCustomSSH(t *testing.T) {
 	assert.Equal(t, "--oProxyCommand=gnome-calculator", e.SSHMetadata.UserAndHost)
 	assert.Equal(t, "/repo", e.SSHMetadata.Path)
 
-	exe, args, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, "git-lfs-authenticate", "download", false)
+	exe, args, _, _ := ssh.GetLFSExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, "git-lfs-authenticate", "download", false, "")
 	assert.Equal(t, "not-ssh", exe)
 	assert.Equal(t, []string{"oProxyCommand=gnome-calculator", "git-lfs-authenticate /repo download"}, args)
 }
@@ -517,7 +560,7 @@ func TestSSHGetExeAndArgsInvalidOptionsAsHost(t *testing.T) {
 	assert.Equal(t, "-oProxyCommand=gnome-calculator", e.SSHMetadata.UserAndHost)
 	assert.Equal(t, "", e.SSHMetadata.Path)
 
-	exe, args, needShell, _ := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, false)
+	exe, args, needShell, _, _ := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, false, "")
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{"--", "-oProxyCommand=gnome-calculator"}, args)
 	assert.Equal(t, false, needShell)
@@ -536,7 +579,7 @@ func TestSSHGetExeAndArgsInvalidOptionsAsPath(t *testing.T) {
 	assert.Equal(t, "git@git-host.com", e.SSHMetadata.UserAndHost)
 	assert.Equal(t, "/-oProxyCommand=gnome-calculator", e.SSHMetadata.Path)
 
-	exe, args, needShell, _ := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, false)
+	exe, args, needShell, _, _ := ssh.GetExeAndArgs(cli.OSEnv(), cli.GitEnv(), &e.SSHMetadata, false, "")
 	assert.Equal(t, "ssh", exe)
 	assert.Equal(t, []string{"git@git-host.com"}, args)
 	assert.Equal(t, false, needShell)
