@@ -38,7 +38,7 @@ begin_test "clone"
   }
   ]" | lfstest-testutils addcommits
 
-  git push origin master
+  git push origin main
 
   # Now clone again, test specific clone dir
   cd "$TRASHDIR"
@@ -116,7 +116,7 @@ begin_test "cloneSSL"
   }
   ]" | lfstest-testutils addcommits
 
-  git push origin master
+  git push origin main
 
   # Now SSL clone again with 'git lfs clone', test specific clone dir
   cd "$TRASHDIR"
@@ -150,6 +150,11 @@ end_test
 begin_test "clone ClientCert"
 (
   set -e
+  if $TRAVIS; then
+    echo "Skipping SSL tests, Travis has weird behaviour in validating custom certs, test locally only"
+    exit 0
+  fi
+
   reponame="test-cloneClientCert"
   setup_remote_repo "$reponame"
   clone_repo_clientcert "$reponame" "$reponame"
@@ -176,30 +181,37 @@ begin_test "clone ClientCert"
   }
   ]" | lfstest-testutils addcommits
 
-  git push origin master
+  git push origin main
 
   # Now clone again with 'git lfs clone', test specific clone dir
   cd "$TRASHDIR"
 
-  newclonedir="testcloneClietCert1"
-  git lfs clone "$CLIENTCERTGITSERVER/$reponame" "$newclonedir" 2>&1 | tee lfsclone.log
-  grep "Cloning into" lfsclone.log
-  grep "Git LFS:" lfsclone.log
-  # should be no filter errors
-  [ ! $(grep "filter" lfsclone.log) ]
-  [ ! $(grep "error" lfsclone.log) ]
-  # should be cloned into location as per arg
-  [ -d "$newclonedir" ]
+  for i in "$LFS_CLIENT_KEY_FILE" "$LFS_CLIENT_KEY_FILE_ENCRYPTED"
+  do
+    export GIT_SSL_CERT_PASSWORD_PROTECTED=1
+    git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "$i"
 
-  # check a few file sizes to make sure pulled
-  pushd "$newclonedir"
-    [ $(wc -c < "file1.dat") -eq 100 ]
-    [ $(wc -c < "file2.dat") -eq 75 ]
-    [ $(wc -c < "file3.dat") -eq 30 ]
-    assert_hooks "$(dot_git_dir)"
-    assert_clean_status
-  popd
+    newclonedir="testcloneClietCert1"
+    rm -fr "$newclonedir"
+    git lfs clone "$CLIENTCERTGITSERVER/$reponame" "$newclonedir" 2>&1 | tee lfsclone.log
+    grep "Cloning into" lfsclone.log
+    # should be no filter errors
+    [ ! $(grep "filter" lfsclone.log) ]
+    [ ! $(grep "error" lfsclone.log) ]
+    # should be cloned into location as per arg
+    [ -d "$newclonedir" ]
 
+    # check a few file sizes to make sure pulled
+    pushd "$newclonedir"
+      [ $(wc -c < "file1.dat") -eq 100 ]
+      [ $(wc -c < "file2.dat") -eq 75 ]
+      [ $(wc -c < "file3.dat") -eq 30 ]
+      assert_hooks "$(dot_git_dir)"
+      assert_clean_status
+    popd
+  done
+
+  git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "$LFS_CLIENT_KEY_FILE"
 
   # Now check SSL clone with standard 'git clone' and smudge download
   rm -rf "$reponame"
@@ -208,10 +220,59 @@ begin_test "clone ClientCert"
 )
 end_test
 
+begin_test "clone ClientCert with homedir certs"
+(
+  set -e
+
+  # Windows triggers a credential helper problem.
+  if [ $IS_WINDOWS -eq 1 ]; then
+    exit 0
+  fi
+
+  reponame="test-cloneClientCert-homedir"
+
+  cp "$LFS_CLIENT_KEY_FILE" "$HOME/lfs-client-key-file"
+  cp "$LFS_CLIENT_CERT_FILE" "$HOME/lfs-client-cert-file"
+
+  git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "~/lfs-client-key-file"
+  git config --global http.$LFS_CLIENT_CERT_URL/.sslCert "~/lfs-client-cert-file"
+
+  setup_remote_repo "$reponame"
+  clone_repo_clientcert "$reponame" "$reponame"
+  if [ $(grep -c "client-cert-mac-openssl" clone_client_cert.log) -gt 0 ]; then
+    echo "Skipping due to SSL client cert bug in Git"
+    exit 0
+  fi
+
+  git lfs track "*.dat" 2>&1 | tee track.log
+  grep "Tracking \"\*.dat\"" track.log
+
+  # generate some test data & commits with random LFS data
+  echo "[
+  {
+    \"CommitDate\":\"$(get_date -5d)\",
+    \"Files\":[
+      {\"Filename\":\"file1.dat\",\"Size\":100},
+      {\"Filename\":\"file2.dat\",\"Size\":75}]
+  },
+  {
+    \"CommitDate\":\"$(get_date -1d)\",
+    \"Files\":[
+      {\"Filename\":\"file3.dat\",\"Size\":30}]
+  }
+  ]" | lfstest-testutils addcommits
+
+  git push origin main
+)
+end_test
+
 begin_test "clone with flags"
 (
   set -e
 
+  git config --global http.$LFS_CLIENT_CERT_URL/.sslCert "$LFS_CLIENT_CERT_FILE"
+  git config --global http.$LFS_CLIENT_CERT_URL/.sslKey "$LFS_CLIENT_KEY_FILE"
+
   reponame="$(basename "$0" ".sh")-flags"
   setup_remote_repo "$reponame"
   clone_repo "$reponame" "$reponame"
@@ -235,14 +296,14 @@ begin_test "clone with flags"
   },
   {
     \"CommitDate\":\"$(get_date -3d)\",
-    \"ParentBranches\":[\"master\"],
+    \"ParentBranches\":[\"main\"],
     \"Files\":[
       {\"Filename\":\"file3.dat\",\"Size\":120},
       {\"Filename\":\"file4.dat\",\"Size\":30}]
   }
   ]" | lfstest-testutils addcommits
 
-  git push origin master branch2
+  git push origin main branch2
 
   # Now clone again, test specific clone dir
   cd "$TRASHDIR"
@@ -308,17 +369,17 @@ begin_test "clone (with include/exclude args)"
 
   contents_a="a"
   contents_a_oid=$(calc_oid "$contents_a")
-  printf "$contents_a" > "a.dat"
-  printf "$contents_a" > "a-dupe.dat"
-  printf "$contents_a" > "dupe-a.dat"
+  printf "%s" "$contents_a" > "a.dat"
+  printf "%s" "$contents_a" > "a-dupe.dat"
+  printf "%s" "$contents_a" > "dupe-a.dat"
 
   contents_b="b"
   contents_b_oid=$(calc_oid "$contents_b")
-  printf "$contents_b" > "b.dat"
+  printf "%s" "$contents_b" > "b.dat"
 
   git add *.dat .gitattributes
   git commit -m "add a.dat, b.dat" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
   grep "5 files changed" commit.log
   grep "create mode 100644 a.dat" commit.log
   grep "create mode 100644 a-dupe.dat" commit.log
@@ -326,8 +387,8 @@ begin_test "clone (with include/exclude args)"
   grep "create mode 100644 b.dat" commit.log
   grep "create mode 100644 .gitattributes" commit.log
 
-  git push origin master 2>&1 | tee push.log
-  grep "master -> master" push.log
+  git push origin main 2>&1 | tee push.log
+  grep "main -> main" push.log
   grep "Uploading LFS objects: 100% (2/2), 2 B" push.log
 
   cd "$TRASHDIR"
@@ -369,15 +430,15 @@ begin_test "clone (with .lfsconfig)"
 
   contents_a="a"
   contents_a_oid=$(calc_oid "$contents_a")
-  printf "$contents_a" > "a.dat"
+  printf "%s" "$contents_a" > "a.dat"
 
   contents_b="b"
   contents_b_oid=$(calc_oid "$contents_b")
-  printf "$contents_b" > "b.dat"
+  printf "%s" "$contents_b" > "b.dat"
 
   git add a.dat b.dat .gitattributes
   git commit -m "add a.dat, b.dat" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
   grep "3 files changed" commit.log
   grep "create mode 100644 a.dat" commit.log
   grep "create mode 100644 b.dat" commit.log
@@ -386,12 +447,12 @@ begin_test "clone (with .lfsconfig)"
   git config -f ".lfsconfig" "lfs.fetchinclude" "a*"
   git add ".lfsconfig"
   git commit -m "config lfs.fetchinclude a*" 2>&1 | tee commit.log
-  grep "master" commit.log
+  grep "main" commit.log
   grep "1 file changed" commit.log
   grep "create mode 100644 .lfsconfig" commit.log
 
-  git push origin master 2>&1 | tee push.log
-  grep "master -> master" push.log
+  git push origin main 2>&1 | tee push.log
+  grep "main -> main" push.log
   grep "Uploading LFS objects: 100% (2/2), 2 B" push.log
 
   pushd "$TRASHDIR"
@@ -431,10 +492,10 @@ begin_test "clone (with .lfsconfig)"
   git config -f ".lfsconfig" "lfs.fetchexclude" "a*"
   git add .lfsconfig
   git commit -m "config lfs.fetchinclude a*" 2>&1 | tee commit.log
-  grep "master" commit.log
+  grep "main" commit.log
   grep "1 file changed" commit.log
-  git push origin master 2>&1 | tee push.log
-  grep "master -> master" push.log
+  git push origin main 2>&1 | tee push.log
+  grep "main -> main" push.log
 
   pushd "$TRASHDIR"
 
@@ -474,14 +535,14 @@ begin_test "clone (without clean filter)"
 
   contents_a="a"
   contents_a_oid=$(calc_oid "$contents_a")
-  printf "$contents_a" > "a.dat"
+  printf "%s" "$contents_a" > "a.dat"
 
   git add *.dat .gitattributes
   git commit -m "add a.dat, b.dat" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
 
-  git push origin master 2>&1 | tee push.log
-  grep "master -> master" push.log
+  git push origin main 2>&1 | tee push.log
+  grep "main -> main" push.log
   grep "Uploading LFS objects: 100% (1/1), 1 B" push.log
 
   cd "$TRASHDIR"
@@ -527,10 +588,10 @@ begin_test "clone with submodules"
 
   contents_sub2="Inception. Now, before you bother telling me it's impossible..."
   contents_sub2_oid=$(calc_oid "$contents_sub2")
-  printf "$contents_sub2" > "sub2.dat"
+  printf "%s" "$contents_sub2" > "sub2.dat"
   git add sub2.dat .gitattributes
   git commit -m "Nested submodule level 2"
-  git push origin master
+  git push origin main
 
   clone_repo "$submodname1" submod1
   git lfs track "*.dat" 2>&1 | tee track.log
@@ -538,13 +599,13 @@ begin_test "clone with submodules"
 
   contents_sub1="We're dreaming?"
   contents_sub1_oid=$(calc_oid "$contents_sub1")
-  printf "$contents_sub1" > "sub1.dat"
+  printf "%s" "$contents_sub1" > "sub1.dat"
   # add submodule2 as submodule of submodule1
   git submodule add "$GITSERVER/$submodname2" sub2
   git submodule update
   git add sub2 sub1.dat .gitattributes
   git commit -m "Nested submodule level 1"
-  git push origin master
+  git push origin main
 
   clone_repo "$reponame" rootrepo
   git lfs track "*.dat" 2>&1 | tee track.log
@@ -552,13 +613,13 @@ begin_test "clone with submodules"
 
   contents_root="Downwards is the only way forwards."
   contents_root_oid=$(calc_oid "$contents_root")
-  printf "$contents_root" > "root.dat"
+  printf "%s" "$contents_root" > "root.dat"
   # add submodule1 as submodule of root
   git submodule add "$GITSERVER/$submodname1" sub1
   git submodule update
   git add sub1 root.dat .gitattributes
   git commit -m "Root repo"
-  git push origin master
+  git push origin main
 
   pushd "$TRASHDIR"
 
@@ -597,23 +658,24 @@ begin_test "clone in current directory"
   contents="contents"
   contents_oid="$(calc_oid "$contents")"
 
-  printf "$contents" > a.dat
+  printf "%s" "$contents" > a.dat
 
   git add .gitattributes a.dat
 
   git commit -m "initial commit" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
   grep "2 files changed" commit.log
   grep "create mode 100644 a.dat" commit.log
   grep "create mode 100644 .gitattributes" commit.log
 
-  git push origin master 2>&1 | tee push.log
+  git push origin main 2>&1 | tee push.log
 
   pushd $TRASHDIR
     mkdir "$reponame-clone"
     cd "$reponame-clone"
 
-    git lfs clone $GITSERVER/$reponame "." 2>&1 | grep "Downloading LFS objects: 100% (1/1), 8 B"
+    git lfs clone $GITSERVER/$reponame "."
+    git lfs fsck
 
     assert_local_object "$contents_oid" 8
     assert_hooks "$(dot_git_dir)"
@@ -653,3 +715,93 @@ begin_test "clone bare empty repository"
   fi
 )
 end_test
+
+begin_test "clone (HTTP server/proxy require cookies)"
+(
+  set -e
+
+  # golang net.http.Cookie ignores cookies with IP instead of domain/hostname
+  GITSERVER=$(echo "$GITSERVER" | sed 's/127\.0\.0\.1/localhost/')
+  cp "$CREDSDIR/127.0.0.1" "$CREDSDIR/localhost"
+  printf "localhost\tTRUE\t/\tFALSE\t2145916800\tCOOKIE_GITLFS\tsecret\n" >> "$REMOTEDIR/cookies.txt"
+  git config --global http.cookieFile "$REMOTEDIR/cookies.txt"
+
+  reponame="require-cookie-test"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git lfs track "*.dat" 2>&1 | tee track.log
+  grep "Tracking \"\*.dat\"" track.log
+
+  # generate some test data & commits with random LFS data
+  echo "[
+  {
+    \"CommitDate\":\"$(get_date -10d)\",
+    \"Files\":[
+      {\"Filename\":\"file1.dat\",\"Size\":100},
+      {\"Filename\":\"file2.dat\",\"Size\":75}]
+  },
+  {
+    \"CommitDate\":\"$(get_date -7d)\",
+    \"Files\":[
+      {\"Filename\":\"file1.dat\",\"Size\":110},
+      {\"Filename\":\"file3.dat\",\"Size\":66},
+      {\"Filename\":\"file4.dat\",\"Size\":23}]
+  },
+  {
+    \"CommitDate\":\"$(get_date -10d)\",
+    \"Files\":[
+      {\"Filename\":\"file5.dat\",\"Size\":120},
+      {\"Filename\":\"file6.dat\",\"Size\":30}]
+  }
+  ]" | lfstest-testutils addcommits
+
+  git push origin main
+
+  # Now clone again, test specific clone dir
+  cd "$TRASHDIR"
+
+  newclonedir="require-cookie-test1"
+  git lfs clone "$GITSERVER/$reponame" "$newclonedir" 2>&1 | tee lfsclone.log
+  grep "Cloning into" lfsclone.log
+  grep "Downloading LFS objects:" lfsclone.log
+  # should be no filter errors
+  [ ! $(grep "filter" lfsclone.log) ]
+  [ ! $(grep "error" lfsclone.log) ]
+  # should be cloned into location as per arg
+  [ -d "$newclonedir" ]
+
+  # check a few file sizes to make sure pulled
+  pushd "$newclonedir"
+    [ $(wc -c < "file1.dat") -eq 110 ]
+    [ $(wc -c < "file2.dat") -eq 75 ]
+    [ $(wc -c < "file3.dat") -eq 66 ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
+  popd
+
+  # Now check clone with implied dir
+  rm -rf "$reponame"
+  git lfs clone "$GITSERVER/$reponame" 2>&1 | tee lfsclone.log
+  grep "Cloning into" lfsclone.log
+  grep "Downloading LFS objects:" lfsclone.log
+  # should be no filter errors
+  [ ! $(grep "filter" lfsclone.log) ]
+  [ ! $(grep "error" lfsclone.log) ]
+  # clone location should be implied
+  [ -d "$reponame" ]
+
+  pushd "$reponame"
+    [ $(wc -c < "file1.dat") -eq 110 ]
+    [ $(wc -c < "file2.dat") -eq 75 ]
+    [ $(wc -c < "file3.dat") -eq 66 ]
+    assert_hooks "$(dot_git_dir)"
+    [ ! -e "lfs" ]
+    assert_clean_status
+  popd
+
+  # to avoid breaking t-credentials.sh
+  rm "$CREDSDIR/localhost"
+)
+end_test
