@@ -6,21 +6,27 @@ envInitConfig='git config filter.lfs.process = "git-lfs filter-process"
 git config filter.lfs.smudge = "git-lfs smudge -- %f"
 git config filter.lfs.clean = "git-lfs clean -- %f"'
 
+unset_vars() {
+    # If set, these will cause the test to fail.
+    unset GIT_LFS_NO_TEST_COUNT GIT_LFS_LOCK_ACQUIRE_DISABLED
+}
+
 begin_test "env with no remote"
 (
   set -e
   reponame="env-no-remote"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
 
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
   expected=$(printf '%s
 %s
@@ -31,7 +37,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -41,12 +47,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -60,19 +67,20 @@ begin_test "env with origin remote"
 (
   set -e
   reponame="env-origin-remote"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
   git remote add origin "$GITSERVER/env-origin-remote"
 
   endpoint="$GITSERVER/$reponame.git/info/lfs (auth=none)"
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
   expected=$(printf '%s
 %s
 
@@ -83,7 +91,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -93,12 +101,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$endpoint" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -116,6 +125,7 @@ begin_test "env with multiple remotes"
 (
   set -e
   reponame="env-multiple-remotes"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
@@ -124,13 +134,13 @@ begin_test "env with multiple remotes"
 
   endpoint="$GITSERVER/env-origin-remote.git/info/lfs (auth=none)"
   endpoint2="$GITSERVER/env-other-remote.git/info/lfs (auth=none)"
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
   expected=$(printf '%s
 %s
 
@@ -142,7 +152,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -152,12 +162,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$endpoint" "$endpoint2" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -175,19 +186,20 @@ begin_test "env with other remote"
 (
   set -e
   reponame="env-other-remote"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
   git remote add other "$GITSERVER/env-other-remote"
 
   endpoint="$GITSERVER/env-other-remote.git/info/lfs (auth=none)"
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
   expected=$(printf '%s
 %s
@@ -199,7 +211,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -209,12 +221,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$endpoint" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -232,6 +245,7 @@ begin_test "env with multiple remotes and lfs.url config"
 (
   set -e
   reponame="env-multiple-remotes-with-lfs-url"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
@@ -239,26 +253,25 @@ begin_test "env with multiple remotes and lfs.url config"
   git remote add other "$GITSERVER/env-other-remote"
   git config lfs.url "http://foo/bar"
 
-  endpoint="$GITSERVER/env-other-remote.git/info/lfs (auth=none)"
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
   expected=$(printf '%s
 %s
 
 Endpoint=http://foo/bar (auth=none)
-Endpoint (other)=%s
+Endpoint (other)=http://foo/bar (auth=none)
 LocalWorkingDir=%s
 LocalGitDir=%s
 LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -268,15 +281,16 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
-' "$(git lfs version)" "$(git version)" "$endpoint" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
+' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
   actual=$(git lfs env | grep -v "^GIT_EXEC_PATH=")
   contains_same_elements "$expected" "$actual"
 
@@ -291,6 +305,7 @@ begin_test "env with multiple remotes and lfs configs"
 (
   set -e
   reponame="env-multiple-remotes-lfs-configs"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
@@ -300,25 +315,25 @@ begin_test "env with multiple remotes and lfs configs"
   git config remote.origin.lfsurl "http://custom/origin"
   git config remote.other.lfsurl "http://custom/other"
 
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
   expected=$(printf '%s
 %s
 
 Endpoint=http://foo/bar (auth=none)
-Endpoint (other)=http://custom/other (auth=none)
+Endpoint (other)=http://foo/bar (auth=none)
 LocalWorkingDir=%s
 LocalGitDir=%s
 LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -328,12 +343,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -347,27 +363,27 @@ UploadTransfers=basic
 )
 end_test
 
-begin_test "env with multiple remotes and lfs url and batch configs"
+begin_test "env with multiple remotes and batch configs"
 (
   set -e
   reponame="env-multiple-remotes-lfs-batch-configs"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
   git remote add origin "$GITSERVER/env-origin-remote"
   git remote add other "$GITSERVER/env-other-remote"
-  git config lfs.url "http://foo/bar"
   git config lfs.concurrenttransfers 5
-  git config remote.origin.lfsurl "http://custom/origin"
+  git config remote.origin.lfsurl "http://foo/bar"
   git config remote.other.lfsurl "http://custom/other"
 
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
   expected=$(printf '%s
 %s
 
@@ -389,12 +405,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -412,6 +429,7 @@ begin_test "env with .lfsconfig"
 (
   set -e
   reponame="env-with-lfsconfig"
+  unset_vars
 
   git init $reponame
   cd $reponame
@@ -430,13 +448,13 @@ lfsurl = http://foobar:5050/
 concurrenttransfers = 50
 ' > .gitconfig
 
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
   expected=$(printf '%s
 %s
 
@@ -447,7 +465,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -457,12 +475,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -480,19 +499,20 @@ begin_test "env with environment variables"
 (
   set -e
   reponame="env-with-envvars"
+  unset_vars
   git init $reponame
   mkdir -p $reponame/a/b/c
 
-  gitDir=$(native_path "$TRASHDIR/$reponame/.git")
-  workTree=$(native_path "$TRASHDIR/$reponame/a/b")
+  gitDir=$(canonical_path "$TRASHDIR/$reponame/.git")
+  workTree=$(canonical_path "$TRASHDIR/$reponame/a/b")
 
-  localwd=$(native_path "$TRASHDIR/$reponame/a/b")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars="$(GIT_DIR=$gitDir GIT_WORK_TREE=$workTree env | grep "^GIT" | sort)"
+  localwd=$(canonical_path "$TRASHDIR/$reponame/a/b")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars="$(GIT_DIR=$gitDir GIT_WORK_TREE=$workTree env | grep "^GIT_" | sort)"
   expected=$(printf '%s
 %s
 
@@ -502,7 +522,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -512,12 +532,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -541,7 +562,7 @@ UploadTransfers=basic
             | grep -v "^GIT_EXEC_PATH=")
   contains_same_elements "$expected" "$actual4"
 
-  envVars="$(GIT_DIR=$gitDir GIT_WORK_TREE=a/b env | grep "^GIT" | sort)"
+  envVars="$(GIT_DIR=$gitDir GIT_WORK_TREE=a/b env | grep "^GIT_" | sort)"
 
   # `a/b` is an invalid relative path from where we are now and results in an
   # error, so resulting output will have many fields blank or invalid
@@ -556,7 +577,7 @@ LocalGitStorageDir=
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -566,12 +587,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=lfs
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 git config filter.lfs.process = ""
 git config filter.lfs.smudge = ""
@@ -582,7 +604,7 @@ git config filter.lfs.clean = ""
   contains_same_elements "$expected5" "$actual5"
 
   cd $TRASHDIR/$reponame/a/b
-  envVars="$(GIT_DIR=$gitDir env | grep "^GIT" | sort)"
+  envVars="$(GIT_DIR=$gitDir env | grep "^GIT_" | sort)"
   expected7=$(printf '%s
 %s
 
@@ -592,7 +614,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -602,12 +624,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -615,7 +638,7 @@ UploadTransfers=basic
   contains_same_elements "$expected7" "$actual7"
 
   cd $TRASHDIR/$reponame/a
-  envVars="$(GIT_WORK_TREE=$workTree env | grep "^GIT" | sort)"
+  envVars="$(GIT_WORK_TREE=$workTree env | grep "^GIT_" | sort)"
   expected8=$(printf '%s
 %s
 
@@ -625,7 +648,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -635,12 +658,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -653,15 +677,16 @@ begin_test "env with bare repo"
 (
   set -e
   reponame="env-with-bare-repo"
+  unset_vars
   git init --bare $reponame
   cd $reponame
 
-  localgit=$(native_path "$TRASHDIR/$reponame")
-  localgitstore=$(native_path "$TRASHDIR/$reponame")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localgit=$(canonical_path "$TRASHDIR/$reponame")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
   expected=$(printf "%s\n%s\n
 LocalWorkingDir=
@@ -670,7 +695,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -680,12 +705,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 " "$(git lfs version)" "$(git version)" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -699,6 +725,7 @@ begin_test "env with multiple ssh remotes"
 (
   set -e
   reponame="env-with-ssh"
+  unset_vars
   mkdir $reponame
   cd $reponame
   git init
@@ -725,20 +752,20 @@ begin_test "env with skip download errors"
 
   git config lfs.skipdownloaderrors 1
 
-  localgit=$(native_path "$TRASHDIR/$reponame")
-  localgitstore=$(native_path "$TRASHDIR/$reponame")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localgit=$(canonical_path "$TRASHDIR/$reponame")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
   expectedenabled=$(printf '%s
 %s
@@ -749,7 +776,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=true
@@ -759,12 +786,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -782,7 +810,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -792,12 +820,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -805,7 +834,7 @@ UploadTransfers=basic
   contains_same_elements "$expecteddisabled" "$actual"
 
   # now enable via env var
-  envVarsEnabled=$(printf "%s" "$(GIT_LFS_SKIP_DOWNLOAD_ERRORS=1 env | grep "^GIT")")
+  envVarsEnabled=$(printf "%s" "$(GIT_LFS_SKIP_DOWNLOAD_ERRORS=1 env | grep "^GIT_")")
   expectedenabled2=$(printf '%s
 %s
 
@@ -815,7 +844,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=false
 BasicTransfersOnly=false
 SkipDownloadErrors=true
@@ -825,12 +854,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic
-UploadTransfers=basic
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVarsEnabled" "$envInitConfig")
@@ -847,26 +877,27 @@ begin_test "env with extra transfer methods"
 (
   set -e
   reponame="env-with-transfers"
+  unset_vars
   git init $reponame
   cd $reponame
 
   git config lfs.tustransfers true
   git config lfs.customtransfer.supertransfer.path /path/to/something
 
-  localgit=$(native_path "$TRASHDIR/$reponame")
-  localgitstore=$(native_path "$TRASHDIR/$reponame")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localgit=$(canonical_path "$TRASHDIR/$reponame")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
-  localwd=$(native_path "$TRASHDIR/$reponame")
-  localgit=$(native_path "$TRASHDIR/$reponame/.git")
-  localgitstore=$(native_path "$TRASHDIR/$reponame/.git")
-  lfsstorage=$(native_path "$TRASHDIR/$reponame/.git/lfs")
-  localmedia=$(native_path "$TRASHDIR/$reponame/.git/lfs/objects")
-  tempdir=$(native_path "$TRASHDIR/$reponame/.git/lfs/tmp")
-  envVars=$(printf "%s" "$(env | grep "^GIT")")
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
 
   expectedenabled=$(printf '%s
 %s
@@ -877,7 +908,7 @@ LocalGitStorageDir=%s
 LocalMediaDir=%s
 LocalReferenceDirs=
 TempDir=%s
-ConcurrentTransfers=3
+ConcurrentTransfers=8
 TusTransfers=true
 BasicTransfersOnly=false
 SkipDownloadErrors=false
@@ -887,12 +918,13 @@ FetchRecentCommitsDays=0
 FetchRecentRefsIncludeRemotes=true
 PruneOffsetDays=3
 PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
 PruneRemoteName=origin
 LfsStorageDir=%s
 AccessDownload=none
 AccessUpload=none
-DownloadTransfers=basic,supertransfer
-UploadTransfers=basic,supertransfer,tus
+DownloadTransfers=basic,lfs-standalone-file,ssh,supertransfer
+UploadTransfers=basic,lfs-standalone-file,ssh,supertransfer,tus
 %s
 %s
 ' "$(git lfs version)" "$(git version)" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
@@ -901,3 +933,216 @@ UploadTransfers=basic,supertransfer,tus
 
 )
 end_test
+
+begin_test "env with multiple remotes and ref"
+(
+  set -e
+  reponame="env-multiple-remotes-ref"
+  unset_vars
+  mkdir $reponame
+  cd $reponame
+  git init
+  git remote add origin "$GITSERVER/env-origin-remote"
+  git remote add other "$GITSERVER/env-other-remote"
+
+  touch a.txt
+  git add a.txt
+  git commit -m "initial commit"
+
+  endpoint="$GITSERVER/env-origin-remote.git/info/lfs (auth=none)"
+  endpoint2="$GITSERVER/env-other-remote.git/info/lfs (auth=none)"
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
+  expected=$(printf '%s
+%s
+
+Endpoint=%s
+Endpoint (other)=%s
+LocalWorkingDir=%s
+LocalGitDir=%s
+LocalGitStorageDir=%s
+LocalMediaDir=%s
+LocalReferenceDirs=
+TempDir=%s
+ConcurrentTransfers=8
+TusTransfers=false
+BasicTransfersOnly=false
+SkipDownloadErrors=false
+FetchRecentAlways=false
+FetchRecentRefsDays=7
+FetchRecentCommitsDays=0
+FetchRecentRefsIncludeRemotes=true
+PruneOffsetDays=3
+PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
+PruneRemoteName=origin
+LfsStorageDir=%s
+AccessDownload=none
+AccessUpload=none
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
+%s
+%s
+' "$(git lfs version)" "$(git version)" "$endpoint" "$endpoint2" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
+  actual=$(git lfs env | grep -v "^GIT_EXEC_PATH=")
+  contains_same_elements "$expected" "$actual"
+)
+end_test
+
+
+begin_test "env with unicode"
+(
+  set -e
+  # This contains a Unicode apostrophe, an E with grave accent, and a Euro sign.
+  # Only the middle one is representable in ISO-8859-1.
+  reponame="env-d’autre-nom-très-bizarr€"
+  unset_vars
+  mkdir $reponame
+  cd $reponame
+  git init
+  git remote add origin "$GITSERVER/env-origin-remote"
+  git remote add other "$GITSERVER/env-other-remote"
+
+  touch a.txt
+  git add a.txt
+  git commit -m "initial commit"
+
+  # Set by the testsuite.
+  unset LC_ALL
+
+  endpoint="$GITSERVER/env-origin-remote.git/info/lfs (auth=none)"
+  endpoint2="$GITSERVER/env-other-remote.git/info/lfs (auth=none)"
+  localwd=$(canonical_path "$TRASHDIR/$reponame")
+  localgit=$(canonical_path "$TRASHDIR/$reponame/.git")
+  localgitstore=$(canonical_path "$TRASHDIR/$reponame/.git")
+  lfsstorage=$(canonical_path "$TRASHDIR/$reponame/.git/lfs")
+  localmedia=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/objects")
+  tempdir=$(canonical_path "$TRASHDIR/$reponame/.git/lfs/tmp")
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
+  expected=$(printf '%s
+%s
+
+Endpoint=%s
+Endpoint (other)=%s
+LocalWorkingDir=%s
+LocalGitDir=%s
+LocalGitStorageDir=%s
+LocalMediaDir=%s
+LocalReferenceDirs=
+TempDir=%s
+ConcurrentTransfers=8
+TusTransfers=false
+BasicTransfersOnly=false
+SkipDownloadErrors=false
+FetchRecentAlways=false
+FetchRecentRefsDays=7
+FetchRecentCommitsDays=0
+FetchRecentRefsIncludeRemotes=true
+PruneOffsetDays=3
+PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
+PruneRemoteName=origin
+LfsStorageDir=%s
+AccessDownload=none
+AccessUpload=none
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
+%s
+%s
+' "$(git lfs version)" "$(git version)" "$endpoint" "$endpoint2" "$localwd" "$localgit" "$localgitstore" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
+  actual=$(git lfs env | grep -v "^GIT_EXEC_PATH=")
+  contains_same_elements "$expected" "$actual"
+)
+end_test
+
+begin_test "env outside a repository"
+(
+  set -e
+  unset_vars
+
+  # This may or may not work, depending on the system, but it should at least
+  # potentially cause Git to print non-English messages.
+  export LC_ALL=fr_FR.UTF-8
+
+  localmedia="$(native_path "lfs/objects")"
+  lfsstorage=lfs
+  tempdir="$(native_path "lfs/tmp")"
+  envVars=$(printf "%s" "$(env | grep "^GIT_")")
+  expected=$(printf '%s
+%s
+
+LocalWorkingDir=
+LocalGitDir=
+LocalGitStorageDir=
+LocalMediaDir=%s
+LocalReferenceDirs=
+TempDir=%s
+ConcurrentTransfers=8
+TusTransfers=false
+BasicTransfersOnly=false
+SkipDownloadErrors=false
+FetchRecentAlways=false
+FetchRecentRefsDays=7
+FetchRecentCommitsDays=0
+FetchRecentRefsIncludeRemotes=true
+PruneOffsetDays=3
+PruneVerifyRemoteAlways=false
+PruneVerifyUnreachableAlways=false
+PruneRemoteName=origin
+LfsStorageDir=%s
+AccessDownload=none
+AccessUpload=none
+DownloadTransfers=basic,lfs-standalone-file,ssh
+UploadTransfers=basic,lfs-standalone-file,ssh
+%s
+%s
+' "$(git lfs version)" "$(git version)" "$localmedia" "$tempdir" "$lfsstorage" "$envVars" "$envInitConfig")
+  # We redirect the standard error here because we should not get any error
+  # messages, and if we do, we want to fail.
+  actual=$(git lfs env 2>&1 | grep -v "^GIT_EXEC_PATH=")
+  contains_same_elements "$expected" "$actual"
+)
+end_test
+
+begin_test "env with duplicate endpoints"
+(
+  set -e
+  reponame="env-duplicate-endpoints"
+  unset_vars
+  mkdir $reponame
+  cd $reponame
+  git init
+  git remote add origin "$GITSERVER/env-origin-remote"
+  git remote add other "$GITSERVER/env-other-remote"
+
+  touch a.txt
+  git add a.txt
+  git commit -m "initial commit"
+
+  cat <<EOF >>.git/config
+[url "https://host.example/domain/"]
+	insteadOf = git@host.example:domain/
+[url "https://host.example/domain/"]
+	insteadOf = git@host.example:domain/
+EOF
+
+  git lfs env 2>&1 | tee test.log
+  if grep 'warning.*same alias' test.log
+  then
+    exit 1
+  fi
+
+  cat <<EOF >>.git/config
+[url "https://somewhere-else.example/domain/"]
+	insteadOf = git@host.example:domain/
+EOF
+
+  git lfs env 2>&1 | tee test.log
+  grep 'warning.*same alias' test.log
+)
+end_test
